/*-------------------------------------------------------------------------*
 *
 * Mobitex.h
 *  Contains prototypes and structure for Mobitex specific Radio API.
 *
 * Copyright 1997-2000 Research In Motion Limited.  All Rights Reserved
 *
 *-------------------------------------------------------------------------*/

/* Unused portions removed Dec. 7, 2001, jgrand */

#ifndef __RADIO_H
#define __RADIO_H

#pragma pack(push,1) // align to 1-byte boundaries


/*-------------------------------------------------------------------------*
 * Types.
 *-------------------------------------------------------------------------*/

/* Time Type (from Rim.h) */
typedef struct              // USED FOR GET/SET DATE and TIME
{
    unsigned char    second;
    unsigned char    minute;
    unsigned char    hour;
    unsigned char    date;
    unsigned char    month;
    unsigned char    day;            // day of week, Sun=0, Mon=1, ..., Sat=6
    unsigned short   year;
    unsigned char    TimeZone;
} TIME;

// MPAK_HEADER structure.  The OS places parsed MPAK header info into this
// structure, and uses this structure to build and MPAK.

typedef struct {
    long  Sender;               // sender MAN number
    long  Destination;          // Addressee MAN number
    int   MpakType;             // PSUBCOM mpak type, 0 for non PSUBCOM
    int   HPID;                 // MPAK HPID for HPDATA Mpaks
    int   Flags;                // MPAK flags component
    TIME  Timestamp;            // parsed Mobitex Time Stamp
    long  lTime;                // MPAK raw Mobitex Time Stamp
    int   TrafficState;         // MPAK traffic state
} MPAK_HEADER;


/*-------------------------------------------------------------------------*
 * Constants.
 *-------------------------------------------------------------------------*/

// MPAK traffic states (in MPAK_HEADER).  These error codes
// are set by the network when an MPAK is routed back to the sender because of problems.

#define  TS_MESSAGE_OK                  0      // MESSAGE OK
#define  TS_MESSAGE_FROM_MAILBOX        1      // MESSAGE FROM MAILBOX
#define  TS_MESSAGE_IN_MAILBOX          2      // MESSAGE WAS PLACED IN MAILBOX & COPY RETURNED
#define  TS_CANNOT_BE_REACHED           3      // RETURNED, CANNOT BE REACHED
#define  TS_ILLEGAL_MESSAGE             4      // RETURNED, BAD MESSAGE FORMAT
#define  TS_NETWORK_CONGESTED           5      // RETURNED, NETWORK CONGESTED
#define  TS_TECHNICAL_ERROR             6      // RETURNED, TECHNICAL NETWORK ERROR
#define  TS_DESTINATION_BUSY            7      // RETURNED, DESTINATION BUSY

// subscription flags (in MPAK_HEADER)
#define  FLAG_MAILBOX        1
#define  FLAG_POSACK         2
#define  FLAG_SENDLIST       4
#define  FLAG_UNKNOWN_FLAG   8

// MPAK packet types (in MPAK_HEADER)
// a value of zero will indicate that the MPAK is not of class PSUBCOM, and
// thus the user must build/parse a raw MPAK
#define  MPAK_TEXT      1
#define  MPAK_DATA      2
#define  MPAK_STATUS    3
#define  MPAK_HPDATA    4


//
// Additional definitions, not absolutely required to use the API
//

#define MAX_MPAK_SIZE       560
#define MAX_MPAK_DATA_SIZE  512

// MPAK classes
#define PCLASS_PSUBCOM  0
#define PCLASS_DTESERV  3
#define PCLASS_PSOSCOM  1  

// DTESERV-class MPAK types
#define MPAK_LOGINREQ   1
#define MPAK_LOGINGRA   2
#define MPAK_LOGINREF   3
#define MPAK_LOGOUT     4
#define MPAK_LOGOUTORD  5
#define MPAK_ACTIVE     7
#define MPAK_INACTIVE   8
#define MPAK_DIE        9
#define MPAK_LIVE       10
#define MPAK_VICESOSRX  13
#define MPAK_SOSRX      14
#define MPAK_GROUPLIST  15
#define MPAK_FLEXLIST   17
#define MPAK_TIME       20
#define MPAK_MODE       24

// assists for parsing MPAKS
#define TRAFFIC_STATE_OFFSET 6
#define TRAFFIC_STATE_MASK   0xe0
#define TRAFFIC_STATE_SHIFT  5
#define TRAFFIC_STATE_OK     0

#define MAILBOX_FLAG_OFFSET 6
#define MAILBOX_FLAG_MASK   0x01
#define MAILBOX_FLAG_SHIFT  0

#define POSACK_FLAG_OFFSET 6
#define POSACK_FLAG_MASK   0x02
#define POSACK_FLAG_SHIFT  1

#define SENDLIST_FLAG_OFFSET 6
#define SENDLIST_FLAG_MASK   0x04
#define SENDLIST_FLAG_SHIFT  2

#define UNKNOWN_FLAG_OFFSET 6
#define UNKNOWN_FLAG_MASK   0x08
#define UNKNOWN_FLAG_SHIFT  3

#define PACKET_CLASS_OFFSET 7
#define PACKET_CLASS_MASK   0xc0
#define PACKET_CLASS_SHIFT  6
#define PACKET_CLASS(raw)   (((raw)[PACKET_CLASS_OFFSET] & PACKET_CLASS_MASK) >> PACKET_CLASS_SHIFT)

#define EXTERNAL_FLAG_OFFSET 7
#define EXTERNAL_FLAG_MASK   0x20
#define EXTERNAL_FLAG_SHIFT  5

#define PACKET_TYPE_OFFSET  7
#define PACKET_TYPE_MASK    0x1f
#define PACKET_TYPE_SHIFT   0
#define PACKET_TYPE(raw)    (((raw)[PACKET_TYPE_OFFSET] & PACKET_TYPE_MASK) >> PACKET_TYPE_SHIFT)

#define INDEPENDENT_HEADER_SIZE 8
#define TIMESTAMP_OFFSET    8
#define TIMESTAMP_LENGTH    3
#define DATA_OFFSET         11
#define SENDLIST_OFFSET     8
#define SENDLIST_LENGTH     22
#define HP_PROTOCOL_ID_OFFSET 11


#pragma pack(pop)

#endif  /* __RADIO_H */

