/*
	util.c
 
    This file is part of munpak.
 
    Utility functions

    Copyright (c) 2002 @stake, Inc.
*/

#include <stdio.h>
#include <windows.h>
#include <string.h>
#include "munpak.h"


void hexdisplay (unsigned char *data, unsigned long length)
{
    unsigned long i, n;

	fprintf(stdout, "\n");

    while( length > 0 )
    {
        n = (length > 18) ? 18 : length;
		for( i = 0; i < n; i++ )
		{
            fprintf(stdout, "%02X ", data[i] );
			if (i == 8) fprintf(stdout, " ");
		}

        // additional spacing for nice looking format
        if (n < 18)
        {
            if (n < 9) fprintf(stdout, " ");
            for (i = 0; i < 18 - n; ++i)
                fprintf(stdout, "   ");
        }

        fprintf(stdout, "  ");
        for( i = 0; i < n; i++, data++ )
            fprintf(stdout, "%c", (*data < 32 || *data > 127) ? '.' : *data );
     
		length -= n;
		if( length > 0 )
			fprintf(stdout, "\n");
	}
}


// Find a string within a given memory range
char *strmem (const char *string, const char *strCharSet, int memlen)
{
    int i;
    char *temp, *result;

    // Make a copy of the data
    temp = malloc(memlen);
    memcpy(temp, string, memlen);
    
    // Change all 0x00 to 0xCC to allow use of strstr
    for (i = 0; i < memlen; ++i)
        if (temp[i] == '\0') temp[i] = (char) 0xCC;

    temp[memlen-1] = '\0';

    result = strstr(temp, strCharSet);
    free (temp);

    return (result);
}


int atoc (const char *instr, char *out)
{
    unsigned char b;
    char c;
    int i;

    i=0;
    b=0;
    while(i<64) {
        b<<=4;
        c=instr[i];
        if(c==0)
            return 0;
        if((c>='0') && (c<='9')) b+=(c-'0');
        else if((c>='a') && (c<='f')) b+=(c-'a'+10);
        else if((c>='A') && (c<='F')) b+=(c-'A'+10);
        if((i%2)==1) {
            out[i/2]=b;
            b=0;
        }
        i++;
    }
  
    return 1;
}


void freelist (DONELIST *pl)
{
	DONELIST *next;

	while (pl != NULL) {
		next = pl->next;
		free (pl);
		pl = next;
	}
}
